var Strut = {

  // Random float between min and max
  random: function(min, max) {
    return Math.random() *  (max - min) + min;
  },

  // Returns a random item from an array
  arrayRandom: function(arr) {
    return arr[ Math.floor( Math.random() * arr.length ) ];
  },

  // Linear interpolation between a and b
  // Ex: (100, 200, 0.5) = 150
  interpolate: function(a, b, i) {
    return a*(1-i) + b*i;
  },

  // Calculate how far i is between a and b
  // Ex: (100, 200, 150) = 0.5
  rangePosition: function(a, b, i) {
    return (i-a) / (b-a);
  },

  // Limits a number on both min and max ends
  clamp: function(num, min, max) {
    return Math.max( Math.min( num, max ), min );
  },

  // querySelectorAll as an array
  queryArray: function(selector, node) {
    if (!node) node = document.body;
    return Array.prototype.slice.call(node.querySelectorAll(selector));
  },

  // Handle DOMContentLoaded after document has already been loaded
  ready: function(fn) {
    if (document.readyState !== 'loading') fn();
    else document.addEventListener('DOMContentLoaded', fn);
  },

  debounce: function (fn, time) {
    let timeout;

    return function() {
      var functionCall = function() {
        return fn.apply(this, arguments);
      }

      clearTimeout(timeout);
      timeout = setTimeout(functionCall, time);
    }
  },

  throttle: function(callback, wait, context) {
    var c = context || this;
    var timeout = null;
    var callbackArgs = null;

    var later = function() {
      callback.apply(c, callbackArgs);
      timeout = null;
    }

    return function() {
      if (!timeout) {
        callbackArgs = arguments;
        timeout = setTimeout(later, wait);
      }
    }
  }

};

function LogoBubbles(conf) {
  var self = this;

  for (i in conf) self[i] = conf[i];

  self.container = document.querySelector(self.containerSelector);
  self.noiseT = 0;
  self.scrollX = 0;

  self.logos.forEach(function(l, i){
    self.logos[i] = {index: i, title: l};
  });

  shuffleArray(self.logos);

  self.vertShrink = 0;
  updateVertShrink();
  window.addEventListener("resize", updateVertShrink);

  self.playing = false;
  updatePlayState();
  window.addEventListener("scroll", updatePlayState);

  // Img preload

  self.logosLoaded = false;

  // Create bubbles

  for (var i = 0; i < self.bubbles.length; i++) {
    var b = self.bubbles[i];
    var logoIndex = i % self.logos.length;

    b.scale = b.s || 1;

    b.seedX = Math.random() * 10000;
    b.seedY = Math.random() * 10000;
    b.noiseX = b.noiseY = 0;

    b.introDelay = Math.random() * self.introDelay;
    b.introProgress = 0;

    b.el = document.createElement("div");
    b.el.className = self.classPrefix + self.logos[logoIndex].index;

    b.tagEl = document.createElement("span");
    b.tagEl.innerHTML = self.logos[logoIndex].title;
    b.el.appendChild(b.tagEl);

    render(b);

    self.container.appendChild(b.el);
  }

  // Functions

  function updateVertShrink() {
    self.vertShrink = reverseInterpolate(1000, 800, window.innerHeight);
    self.vertShrink = clamp(self.vertShrink, 0, 1);
  }

  function updatePlayState() {
    var rect = self.container.getBoundingClientRect();
    if ((rect.bottom < 0 || rect.top > window.innerHeight) && self.playing == true) {
      self.playing = false;
      // dbg.style.backgroundColor = "red";
    } else if (rect.bottom > 0 && rect.top < window.innerHeight && self.playing == false) {
      self.playing = true;
      // dbg.style.backgroundColor = "green";
      requestAnimationFrame(function(t){ self.tick(t) });
    }
  }

  function render(b) {
    var absX = b.x + b.noiseX + self.scrollX;
    var absY = b.y + b.noiseY;
    absY = interpolate(absY, self.containerHeight/2, self.vertShrink*self.maxShrink);

    if (absX < -200) b.x += self.containerWidth;

    var sc = ease( b.introProgress ) / 20 + 0.95;
    sc *= b.scale;

    b.el.style.opacity = ease( b.introProgress );
    b.el.style.transform = "translate("+absX+"px, "+absY+"px) scale("+sc+")";
  }

  function shuffleArray(array) {
    var i = 0, j = 0, temp = null
    for (i = array.length - 1; i > 0; i -= 1) {
      j = Math.floor(Math.random() * (i + 1));
      temp = array[i];
      array[i] = array[j];
      array[j] = temp;
    }
  }

  function ease(t) {
    return t<.5 ? 2*t*t : -1+(4-2*t)*t
  }

  function interpolate(a, b, i) {
    return a*(1-i) + b*i;
  }

  function reverseInterpolate(a, b, i) {
    return (i-a) / (b-a);
  }

  function clamp(num,min,max) {
    return Math.max( Math.min( num, max ), min );
  }

  // Public

  self.firstTick = null;
  self.lastTick = 0;

  self.tick = function(timer) {
    if (!self.firstTick) self.firstTick = timer;
    timer -= self.firstTick;

    var delta = timer - self.lastTick;
    self.lastTick = timer;

    // dbg.innerHTML = timer;

    self.noiseT += delta * self.noiseSpeed;
    self.scrollX -= delta * self.scrollSpeed;

    for (var i = 0; i < self.bubbles.length; i++) {
      var b = self.bubbles[i];

      b.noiseX = noise(b.seedX + self.noiseT) * self.noiseScale - self.noiseScale/2;
      b.noiseY = noise(b.seedY + self.noiseT) * self.noiseScale - self.noiseScale/2;

      if (self.logosLoaded && b.introProgress < 1 && timer > b.introDelay) {
        b.introProgress = Math.min( 1, b.introProgress + (delta / self.introDuration) );
      }

      render(b);
    }

    if (self.playing) requestAnimationFrame(self.tick);
  }
}

'use strict';

var PERLIN_ZWRAPB = 8;
var PERLIN_ZWRAP = 1<<PERLIN_ZWRAPB;
var PERLIN_SIZE = 4095;

var perlin_octaves = 4; // default to medium smooth
var perlin_amp_falloff = 0.5; // 50% reduction/octave

var scaled_cosine = function(i) {
  return 0.5*(1.0-Math.cos(i*Math.PI));
};

var perlin; // will be initialized lazily by noise() or noiseSeed()

var noise = function(x) {
  if (perlin == null) {
    perlin = new Array(PERLIN_SIZE + 1);
    for (var i = 0; i < PERLIN_SIZE + 1; i++) {
      perlin[i] = Math.random();
    }
  }

  if (x<0) { x=-x; }

  var xi=Math.floor(x);
  var xf = x - xi;
  var rxf;

  var r=0;
  var ampl=0.5;

  var n1;

  for (var o=0; o<perlin_octaves; o++) {
    rxf = scaled_cosine(xf);

    n1  = perlin[xi&PERLIN_SIZE];
    n1 += rxf*(perlin[(xi+1)&PERLIN_SIZE]-n1);

    r += n1*ampl;
    ampl *= perlin_amp_falloff;
    xi<<=1;
    xf*=2;

    if (xf>=1.0) { xi++; xf--; }
  }
  return r;
};

Strut.ready(function(){

  Strut.queryArray('.Section-quotePhoto--video').forEach(function(vthumb){

    var vcont = document.getElementById(vthumb.getAttribute('data-video'));
    var vwrapper = vcont.querySelector('.VideoContainer-wrapper');
    var vplayer = vcont.querySelector('video');

    // Returns margins around video player, depending on window size
    function expandedPos() {
      var mh = Strut.interpolate(20, 100, Strut.clamp(Strut.rangePosition(670, 1160, window.innerWidth), 0, 1));
      return { left: mh, right: mh, top: 0, bottom: 0 };
    }

    // Convert a boundingClientRect to CSS-compatible values
    function cssRect(el){
      var rect = el.getBoundingClientRect();
      return {
        top: rect.top,
        left: rect.left,
        right: window.innerWidth - rect.right,
        bottom: window.innerHeight - rect.bottom
      };
    }

    // Set CSS position
    function setPos(el, rect) {
      ['top', 'right', 'bottom', 'left'].forEach(function(attr){
        el.style[attr] = rect[attr] + 'px';
      })
    }

    function showContainer() {
      vcont.style.display = 'block';
      vwrapper.offsetHeight; // Force layout update
      vcont.classList.remove('inactive');
    }

    function hideContainer() {
      vcont.classList.add('inactive');
      vcont.addEventListener('transitionend', function(){
        vcont.style.display = 'none';
      }, {once: true});
    }

    // Show video
    vthumb.addEventListener('click', function(e){
      // Defer to iOS's built-in fullscreen video player on iPhones
      if (navigator.userAgent.match(/(iPhone|iPod)/g)) {
        setPos(vwrapper, expandedPos());
        vplayer.play();
        return;
      }

      vwrapper.classList.add('no-anim');
      setPos(vwrapper, cssRect(vthumb));
      vwrapper.offsetHeight; // Force layout update
      vwrapper.classList.remove('no-anim');
      setPos(vwrapper, expandedPos());

      showContainer();
      vplayer.play();

      // Hide on ESC
      document.body.addEventListener('keyup', function(e){
        if (e.key == 'Escape') {
          setPos(vwrapper, cssRect(vthumb));
          hideContainer();
          vplayer.pause();
        }
      }, {once: true})
    })

    // Hide video
    vcont.addEventListener('click', function(e){
      setPos(vwrapper, cssRect(vthumb));
      hideContainer();
      vplayer.pause();
    })

    // Play/pause
    vplayer.addEventListener('click', function(e){
      e.stopPropagation();
      if (vplayer.paused) vplayer.play();
      else vplayer.pause();
    })

  })

})
